"""Events for the storages."""

from haolib.storages.events.operations import (
    AfterCreateEvent,
    AfterDeleteEvent,
    AfterFilterEvent,
    AfterMapEvent,
    AfterReadEvent,
    AfterReduceEvent,
    AfterTransformEvent,
    AfterUpdateEvent,
    BeforeCreateEvent,
    BeforeDeleteEvent,
    BeforeFilterEvent,
    BeforeMapEvent,
    BeforeReadEvent,
    BeforeReduceEvent,
    BeforeTransformEvent,
    BeforeUpdateEvent,
)

__all__ = [
    "AfterCreateEvent",
    "AfterDeleteEvent",
    "AfterFilterEvent",
    "AfterMapEvent",
    "AfterReadEvent",
    "AfterReduceEvent",
    "AfterTransformEvent",
    "AfterUpdateEvent",
    "BeforeCreateEvent",
    "BeforeDeleteEvent",
    "BeforeFilterEvent",
    "BeforeMapEvent",
    "BeforeReadEvent",
    "BeforeReduceEvent",
    "BeforeTransformEvent",
    "BeforeUpdateEvent",
]
